/**********************************************************************/
/*
     FILE: hcwf.h
  PURPOSE: To define the CoastWatch HDF library interface routines.
   AUTHOR: Peter Hollemans
     DATE: 23/02/1999
  CHANGES: 15/06/1999, PFH, added CWclone
    NOTES: The CoastWatch HDF library consists of a number of special
           functions designed to aid in writing CoastWatch HDF files,
           in cooperation with other HDF routines.  By using these
           functions to create files and variables, an HDF file is
           guaranteed to conform to the CoastWatch HDF Metadata
           Specification.  The functions also help in performing earth
           transform and date calculations, adding entries to a file's
           global history attribute, and reading and writing data.
           Map projection calculations are performed using the General
           Cartographic Transformation Package (GCTP) from the
           USGS National Mapping Division.

           Like most HDF functions, the CoastWatch functions return an
           integer value: SUCCEED (0) if the function finished with no
           errors, or FAIL (-1) if there were errors.  The associated
           error string may be obtained by immediately calling
           CWstrerror.

  CoastWatch Format (CWF) Software Library and Utilities
  Copyright 1998-1999, USDOC/NOAA/NESDIS CoastWatch West Coast Regional Node

*/
/**********************************************************************/

#ifndef HCWF_H
#define HCWF_H

/************************ INCLUDE FILES *******************************/

/* These two include files are for HDF and GCTP.  These are needed so
   that various constants are defined, such as the HDF data type
   constants, GCTP projection system codes, etc. */

#include "mfhdf.h"
#include "proj.h"

/************************ DEFINED MACROS ******************************/

/* The following macros help in setting up the gctp_parm array for
   CWmakeproj. */

#define GCTPPARM(a,p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15) { \
  (a)[0] = (p1); (a)[1] = (p2); (a)[2] = (p3); (a)[3] = (p4); \
  (a)[4] = (p5); (a)[5] = (p6); (a)[6] = (p7); (a)[7] = (p8); \
  (a)[8] = (p9); (a)[9] = (p10); (a)[10] = (p11); (a)[11] = (p12); \
  (a)[12] = (p13); (a)[13] = (p14); (a)[14] = (p15); \
}

#define GCTPPAK(a) (pakr2dm ((a)/57.2957795131))

/************************ TYPE DECLARATIONS ***************************/

/* The following earth transform (xfm) structure holds all information
   necessary for CoastWatch earth transform calculations, including
   all parameters for GCTP.  Refer to the GCTP C package documentation
   for a complete explanation of all GCTP codes.  Normally, this data
   structure can be handled through the CoastWatch projection routines
   CWgetproj, CWmakeproj, etc. */

typedef struct {
  int32 gctp_sys;		/* defines the output projection system */
  int32 gctp_zone;		/* zone for UTM projection system */
  float64 gctp_parm[15];	/* array of projection parameters */
  int32 gctp_datum;		/* spheriod code */
  float64 et_affine[6];		/* affine transform matrix and vector */
  int32 rows, cols;		/* image rows and columns */
} cwxfm_struct;
typedef cwxfm_struct *CWXFM;

/* The date structure encodes the date and time of the data in a
   compact form.  For the convenience of C and UNIX programmers, the
   reference date and time are January 1, 1970, 00:00:00 UTC which is
   what the UNIX system uses internally.  Normally, dates can be
   manipulated using the CWgetdate, CWmakedate, etc. functions.*/

typedef struct {
  int32 pass_date;		/* days since January 1, 1970 */
  float64 start_time;		/* seconds since 00:00:00 UTC */
} cwdate_struct;
typedef cwdate_struct *CWDATE;

/************************ DATASET AND ERROR FUNCTIONS *****************/

intn CWcreate (			/*** CREATE A NEW COASTWATCH HDF FILE ***/
  char *path,			/* file path and name */
  char *sat,			/* satellite name, eg: noaa-14, noaa-15 */
  char *sensor,			/* sensor name, eg: avhrr, seawifs */
  CWXFM xfm,                    /* earth transform obtained from 
                                   CWgetproj or CWmakeproj */
  CWDATE date,			/* date obtained from 
				   CWgetdate or CWmakedate */
  intn day,			/* true if the data is from a daytime 
				   satellite pass, false otherwise */
  int32 *sdidp			/* pointer to HDF SD interface id (returned) */
);

intn CWclone (			/*** CLONE A COASTWATCH HDF FILE ***/
  char *path,			/* file path and name of existing file */
  char *clone,			/* file path and name of clone to create */
  int32 *sdidp			/* pointer to HDF SD interface id (returned) */
);

const char *CWstrerror (	/*** RETURN A DESCRIPTIVE ERROR STRING ***/
  void
);

/************************ EARTH TRANSFORM FUNCTIONS *******************/

/* In the following functions, *xfm is a pointer to a structure that
   is allocated by the routines.  If needed, use free() to explicitly 
   de-allocate the structure memory after use. */

intn CWgetproj (		/*** GET AN EARTH TRANSFORM FROM A FILE ***/
  int32 sdid,			/* HDF SD interface id */
  CWXFM *xfm			/* earth transform (returned) */
);


intn CWmakeproj (		/*** CREATE AN EARTH TRANSFORM ***/
  int32 gctp_sys,		/* defines the output projection system */
  int32 gctp_zone,		/* zone for UTM projection system */
  float64 *gctp_parm,		/* array of projection parameters */
  int32 gctp_datum,		/* spheriod code */
  int32 rows, int32 cols,	/* image rows and columns */
  float64 clat, float64 clon,	/* image center point (lat, lon) in degrees,
                                   positive for north and east, negative for
                                   south and west */
  float64 width,		/* image pixel height at the projection 
				   reference point (meters) */
  float64 height,		/* image pixel width at the projection
				   reference point (meters) */
  CWXFM *xfm			/* earth transform (returned) */
);

intn CWuseproj (		/*** USE EARTH TRANFORM IN CALCULATIONS ***/
  CWXFM xfm			/* earth transform to use in subsequent 
				   calls to CWijtoll and CWlltoij */
);

intn CWsameproj (		/*** COMPARE EARTH TRANSFORMS */
  CWXFM xfma,			/* earth transform A */
  CWXFM xfmb			/* earth transform B */
				/* NOTE: Returns SUCCEED (0) if transforms
				   match, FAIL (-1) if not. */
);

intn CWijtoll (			/*** CONVERT FROM IMAGE TO GEOGRAPHIC ***/
  float64 i, float64 j,		/* image coordinates, i = column, j = row,
				   values are 1-relative */
  float64 *lat, float64 *lon	/* geographic coordinates, lat = latitude,
				   lon = longitude (returned) */
);

intn CWlltoij (			/*** CONVERT FROM GEOGRAPHIC TO IMAGE ***/
  float64 *i, float64 *j,	/* image coordinates, i = column, j = row,
				   values are 1-relative (returned) */
  float64 lat, float64 lon	/* geographic coordinates, lat = latitude,
				   lon = longitude */
);

/************************ DATE FUNCTIONS ******************************/

/* In the following functions, *date is a pointer to a structure that
   is allocated by the routines.  If needed, use free() to explicitly 
   de-allocate the structure memory after use. */

intn CWgetdate (		/*** GET A DATE FROM A FILE ***/
  int32 sdid,			/* HDF SD interface id */
  CWDATE *date			/* date structure (returned) */
);

intn CWmakedate (		/*** CREATE A DATE ***/
  intn year,			/* full year, eg: 1999 */
  intn jday,			/* julian day (1-366) */
  intn hour,			/* hour (0-23) */
  intn minute,			/* minute (0-59) */
  intn second,			/* second (0-59) */
  intn millisecond,		/* millisecond (0-999) */
  CWDATE *date			/* date structure (returned) */
);

void CWdatestr (		/*** CREATE A DESCRIPTIVE DATE STRING ***/
  char *date_str,		/* date string char array (returned) */
  size_t buf,			/* date string array size */
  char *date_format,		/* date format (see strftime(3C)) */
  intn local,			/* true for conversion to local time, 
				   false for UTC time */
  CWDATE date			/* date structure */
);

intn CWcomparedate (		/*** COMPARE DATES ***/
  CWDATE datea,			/* date A */
  CWDATE dateb			/* date B */
				/* NOTE: Returns -1 if A < B, 0 if A = B,
				   or 1 if A > B. */
);

/************************ HISTORY FUNCTIONS ***************************/

intn CWaddhist (		/*** ADD TO GLOBAL HISTORY ATTRIBUTE ***/
  int32 sdid,			/* HDF SD interface id */
  int argc, char *argv[]	/* C program command line parameters */
);

/************************ VARIABLE FUNCTIONS **************************/

/* The following variable creation and read/write routines are
   designed to encode two different types of satellite data variables:

    i) Floating-point values scaled to signed 16-bit integer HDF data
       to hold infrared and visible channels, angles, times, etc.  A
       number of extra attributes are written to the HDF file to help
       with scaling, units, etc.  The 16-bit data values are
       calculated as follows:

         16-bit integer = (float / scale) + offset

       Normally, scale = 0.01 and offset = 0.0 is sufficient.

   ii) Unsigned 8-bit integer (byte) HDF data to hold cloud masks,
       graphics planes, etc.

   NOTE that CWreadfloat and CWreadbyte allocate memory for the data
   before reading it - the user is responsible for freeing the memory.
   Also, care must be taken when manipulating compressed variable
   data:

    i) DO NOT try to write partial data to a compressed data 
       variable - use CWwritefloat or CWwritebyte or write the whole 
       variable with SDwritedata.

   ii) DO NOT try to read back data written to a compressed data
       variable without calling SDendaccess or SDend. */

intn CWnewfloat (		/*** DEFINE NEW SATELLITE FLOAT VARIABLE ***/
  int32 sdid,			/* HDF SD interface id */
  char *sds_name,		/* HDF scientific dataset variable name */
  char *long_name,		/* descriptive variable name */
  char *units,			/* variable units */
  float64 scale,		/* calibration scale */
  float64 offset,		/* calibration offset */
  intn compress,		/* true to turn on GZIP compression, false
				   for no compression */
  int32 *sdsidp			/* pointer to HDF SD dataset id (returned) */
);

intn CWreadfloat (		/*** READ SATELLITE VARIABLE DATA ***/
  int32 sdsid,			/* HDF SD dataset id */
  float32 badval,		/* missing value for uncalibrated data */
  float32 **fp			/* pointer to uncalibrated data array (memory 
				   will be allocated for the array) */
);

intn CWwritefloat (		/*** WRITE SATELLITE VARIABLE DATA ***/
  int32 sdsid,			/* HDF SD dataset id */
  float32 badval,		/* missing value for uncalibrated data */
  float32 *fp			/* uncalibrated data array */
);

intn CWnewbyte (		/*** DEFINE NEW SATELLITE BYTE VARIABLE ***/
  int32 sdid,			/* HDF SD interface id */
  char *sds_name,		/* HDF scientific dataset variable name */
  char *long_name,		/* descriptive variable name */
  intn compress,		/* true to turn on GZIP compression, false
				   for no compression */
  int32 *sdsidp			/* pointer to HDF SD dataset id (returned) */
);

intn CWreadbyte (		/*** READ SATELLITE BYTE DATA ***/
  int32 sdsid,			/* HDF SD dataset id */
  uint8 **bp			/* pointer to byte array (memory 
				   will be allocated for the array) */
);

intn CWwritebyte (		/*** WRITE SATELLITE BYTE DATA ***/
  int32 sdsid,			/* HDF SD dataset id */
  uint8 *bp			/* byte array */
);

#endif

/**********************************************************************/
